/*!
 * \file container/range.h
 * \brief
 *      A plain definition of a range struct with agregate initialization
 *      and begin-end pairs.
 *
 * \copyright Copyright (C) 2021 Christos Choutouridis <christos@choutouridis.net>
 *
 * <dl class=\"section copyright\"><dt>License</dt><dd>
 * The MIT License (MIT)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * </dd></dl>
 */

#ifndef utl_container_range_h__
#define utl_container_range_h__

#include <utl/core/impl.h>

namespace utl {

/*!
 * \brief
 *  A plain definition of a range struct with begin-end pairs.
 *
 * \tparam Iter_t   The iterator type of the range
 */
template <typename Iter_t>
struct range {
    Iter_t  b{}, e{};

//    range () = default;
//    range (const Iter_t& first, const Iter_t& last) noexcept :
//        b(first), e(last) { }
//    range (Iter_t first, Iter_t last) noexcept :
//        b(first), e(last) { }

    Iter_t begin()              { return b; }
    const Iter_t begin() const  { return b; }
    const Iter_t cbegin() const { return b; }
    Iter_t end()                { return e; }
    const Iter_t end() const    { return e; }
    const Iter_t cend() const   { return e; }
};

} // namespace utl;
#endif /* utl_container_range_h__ */
